package be.iminds.ilabt.jfed.experimenter_gui.config;

import be.iminds.ilabt.jfed.fedmon.webapi.service.json.UserInfo;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.UserInfoBuilder;
import be.iminds.ilabt.jfed.preferences.CorePreferenceKey;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.util.common.GeniUrn;
import be.iminds.ilabt.jfed.util.library.JFedVersionInfo;

import javax.inject.Inject;
import javax.inject.Singleton;
import java.util.*;

/**
 * User: twalcari
 * Date: 4/25/2016
 * Time: 14:52
 */
@Singleton
public class UserInfoProvider {
    private final JFedVersionInfo jFedVersionInfo;
    private List<String> extraFlags;

    @Inject
    public UserInfoProvider(JFedVersionInfo jFedVersionInfo, JFedPreferences jFedPreferences) {
        this.jFedVersionInfo = jFedVersionInfo;

        extraFlags = Collections.unmodifiableList(jFedPreferences.getStringList(CorePreferenceKey.PREF_GUICONFIG_EXTRA_FLAGS));
    }

    public void addExtraFlags(String... extraExtraFlags) {
        addExtraFlags(Arrays.asList(extraExtraFlags));
    }
    public void addExtraFlags(List<String> extraExtraFlags) {
        Set<String> res = new TreeSet<>(extraFlags);
        res.addAll(extraExtraFlags);
        extraFlags = Collections.unmodifiableList(new ArrayList<>(res));
    }

    public UserInfo createUserInfo(GeniUrn userUrn, String... extraExtraFlags){
        Set<String> usedExtraFlags = new TreeSet<>(extraFlags);
        usedExtraFlags.addAll(Arrays.asList(extraExtraFlags));

        UserInfoBuilder userInfoBuilder = new UserInfoBuilder();
        userInfoBuilder.setUrn(userUrn);

        userInfoBuilder.setVersionJava(System.getProperty("java.version"));
        userInfoBuilder.setVersionJavaVendor(System.getProperty("java.vendor"));
        userInfoBuilder.setVersionJavaSpec(System.getProperty("java.specification.version"));

        userInfoBuilder.setVersionOs(System.getProperty("os.name") + " " + System.getProperty("os.version"));
        userInfoBuilder.setVersionArch(System.getProperty("os.arch"));

        userInfoBuilder.setVersionJFed(jFedVersionInfo.getFullVersionString());

        userInfoBuilder.setExtra(new ArrayList<>(usedExtraFlags));
        return userInfoBuilder.create();
    }


}
