package be.iminds.ilabt.jfed.experimenter_gui.slice.ribbon_tabs;

import be.iminds.ilabt.jfed.experiment.Experiment;
import be.iminds.ilabt.jfed.experimenter_gui.config.JFedGuiConfig;
import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentViewController;
import be.iminds.ilabt.jfed.experimenter_gui.slice.events.*;
import be.iminds.ilabt.jfed.experimenter_gui.ui.ribbon.RibbonButton;
import be.iminds.ilabt.jfed.gui_preferences.HLPreferenceKey;
import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.gui_preferences.JFedHLPreferences;
import be.iminds.ilabt.jfed.rspec.model.ModelRspec;
import be.iminds.ilabt.jfed.rspec.rspec_source.ManifestRspecSource;
import com.google.common.eventbus.EventBus;
import javafx.beans.value.ObservableValue;
import javafx.beans.value.ObservableValueBase;
import javafx.fxml.FXML;
import javafx.scene.control.Dialog;
import org.reactfx.value.Val;

import java.util.Objects;

/**
 * User: twalcari
 * Date: 7/1/2014
 * Time: 17:20
 */
public class AdvancedSliceRibbonComponentGroupController {
    private final SliceRibbonTab sliceRibbonTab;
    private final EventBus eventBus;
    private final JFedGuiConfig config;

    @FXML
    private RibbonButton restartExperimentButton;
    @FXML
    private RibbonButton editSshKeysExperimentButton;
    @FXML
    private RibbonButton shareSliceButton;
    @FXML
    private RibbonButton unshareSliceButton;
    @FXML
    private RibbonButton linkTestButton;
    @FXML
    private RibbonButton rerunEspecButton;
    @FXML
    private RibbonButton runSshCommandButton;

    private final boolean isDemo = new JFedHLPreferences().getBoolean(HLPreferenceKey.PREF_IS_DEMO, false);

    public AdvancedSliceRibbonComponentGroupController(SliceRibbonTab sliceRibbonTab,
                                                       EventBus eventBus,
                                                       JFedGuiConfig config) {
        this.sliceRibbonTab = sliceRibbonTab;
        this.eventBus = eventBus;
        this.config = config;


    }

    @FXML
    protected void initialize() {
        if (!isDemo && !config.isExperimentActionDisabled()) {
            final Val<Boolean> manifestUnavailableVal =
                    Val.map(sliceRibbonTab.activeExperimentViewControllerProperty(), ExperimentViewController::getExperiment)
                            .flatMap(Experiment::sliceProperty)
                            .flatMap(Slice::manifestRspecProperty)
                            .map(Objects::isNull).orElseConst(true);

            restartExperimentButton.disableProperty().bind(manifestUnavailableVal);
            editSshKeysExperimentButton.disableProperty().bind(manifestUnavailableVal);
            shareSliceButton.disableProperty().bind(manifestUnavailableVal);
            unshareSliceButton.disableProperty().bind(manifestUnavailableVal);

            final ObservableValue<Boolean> hasNoLinksVal = new ObservableValueBase<Boolean>() {
                {
                    sliceRibbonTab.activeExperimentViewControllerProperty().addListener((ob, o, n) -> fireValueChangedEvent());
                    manifestUnavailableVal.addListener((ob, o, n) -> fireValueChangedEvent());
                }

                @Override
                public Boolean getValue() {
                    ExperimentViewController evc = sliceRibbonTab
                            .activeExperimentViewControllerProperty()
                            .getValue();
                    if (evc == null)
                        return true;
                    Slice s = evc
                            .getExperiment()
                            .getSliceOrNull();
                    if (s == null)
                        return true;
                    ManifestRspecSource mrs =
                            s.getManifestRspec();
                    if (mrs == null)
                        return true;
                    ModelRspec mr = mrs.getModelRspec();
                    if (mr == null)
                        return true;
                    return mr.getLinks().isEmpty();
                }
            };
            linkTestButton.disableProperty().bind(hasNoLinksVal);
        } else {
            restartExperimentButton.setDisable(true);
            editSshKeysExperimentButton.setDisable(true);
            shareSliceButton.setDisable(true);
            unshareSliceButton.setDisable(true);
            linkTestButton.setDisable(true);
        }
    }


    @FXML
    private void onRebootExperimentAction() {
        eventBus.post(new RebootExperimentEvent(restartExperimentButton.getScene().getWindow(),
                sliceRibbonTab.getActiveExperimentViewController().getExperimentController()));
    }

    @FXML
    private void onEditSshKeysExperimentAction() {
        eventBus.post(new EditSshKeysEvent(editSshKeysExperimentButton.getScene().getWindow(),
                sliceRibbonTab.getActiveExperimentViewController().getExperimentController()));
    }

    @FXML
    private void onShareSliceAction() {
        eventBus.post(new ShareExperimentEvent(shareSliceButton.getScene().getWindow(),
                sliceRibbonTab.getActiveExperimentViewController().getExperimentController()));
    }

    @FXML
    private void onUnshareSliceAction() {
        eventBus.post(new UnshareExperimentEvent(unshareSliceButton.getScene().getWindow(),
                sliceRibbonTab.getActiveExperimentViewController().getExperimentController()));
    }

    @FXML
    private void onTestLinksAction() {
        eventBus.post(new TestLinkEvent(linkTestButton.getScene().getWindow(),
                sliceRibbonTab.getActiveExperimentViewController().getExperimentController()));
    }

    @FXML
    private void onRerunEspecAction() {
        eventBus.post(new RerunEspecEvent(rerunEspecButton.getScene().getWindow(),
                sliceRibbonTab.getActiveExperimentViewController().getExperimentController()));
    }

    @FXML
    private void onRunSshCommandAction() {
        eventBus.post(new RunSshCommandEvent(runSshCommandButton.getScene().getWindow(),
                sliceRibbonTab.getActiveExperimentViewController().getExperimentController()));
    }
}
